from django.conf import settings

# The minimum time between two counted visits (in minutes)
MIN_TIME_BETWEEN_VISITS = getattr(settings, 'MIN_TIME_BETWEEN_VISITS', 24 * 60)
IGNORE_URLS = getattr(settings, 'IGNORE_URLS', {})
IGNORE_USER_AGENTS = getattr(settings, 'IGNORE_USER_AGENTS', [])
BOTS_USER_AGENTS = getattr(
        settings,
        'BOTS_USER_AGENTS',
        [
                "Teoma", "alexa", "froogle", "Gigabot", "inktomi", "looksmart", "URL_Spider_SQL", "Firefly",
                "NationalDirectory", "Ask Jeeves", "TECNOSEEK", "InfoSeek", "WebFindBot", "girafabot", "crawler",
                "www.galaxy.com", "Googlebot", "Googlebot/2.1", "Google", "Webmaster", "Scooter", "James Bond",
                "Slurp", "msnbot", "appie", "FAST", "WebBug", "Spade", "ZyBorg", "rabaz", "Baiduspider",
                "Feedfetcher-Google", "TechnoratiSnoop", "Rankivabot", "Mediapartners-Google", "Sogou web spider",
                "WebAlta Crawler", "MJ12bot", "Yandex/", "YaDirectBot", "StackRambler", "DotBot", "dotbot"
        ]
)

REQUEST_FIELDS_FOR_HASH = getattr(settings, 'REQUEST_FIELDS_FOR_HASH', ['REMOTE_ADDR', 'HTTP_USER_AGENT'])
URI_WITH_GET_PARAMS = getattr(settings, 'URI_WITH_GET_PARAMS', False)
VISITS_OBJECTS_AS_COUNTERS = getattr(settings, 'VISITS_OBJECTS_AS_COUNTERS', True)



# -*- coding: utf-8 -*-
from django.db import models
from django.db.models.aggregates import Max, Min, Avg
from django.utils.translation import ugettext_lazy as _
from visits.utils import is_ignored, gen_hash
from visits import settings

try:
    from django.utils.timezone import now
except ImportError:
    from datetime import datetime
    now = datetime.now

MAX = 1
AVG = 2
MIN = 3

class VisitManager(models.Manager):
    def get_uri_visits_for(self, request, app_model=None, uri=None):
        if uri:
            return self.filter(
                visitor_hash=gen_hash(request, uri),
                uri=uri,
                ip_address=request.META.get('REMOTE_ADDR','')
            )

        elif app_model:
            return self.filter(
                object_app=app_model._meta.app_label,
                object_model=app_model.__class__.__name__,
                uri__regex="^(.*/){1,}"
            )
        else:
            raise ValueError('You must pass "app_model" or "uri" parameter.')

    def get_object_visits_for(self, app_model=None, obj=None):
        if obj:
            return self.filter(
                object_app=obj._meta.app_label,
                object_model=obj.__class__.__name__,
                object_id=obj.id
            )

        return self.filter(
            object_app=app_model._meta.app_label,
            object_model=app_model.__class__.__name__
        )
 
    def add_uri_visit(self, request, uri, app_label):
        visitor_hash = gen_hash(request, uri)
        if settings.VISITS_OBJECTS_AS_COUNTERS:
            visit = self.get_or_create(
                ip_address=request.META.get('REMOTE_ADDR',''),
                visitor_hash=visitor_hash,
                uri=uri,
                object_app=app_label
            )
        else:
            visit = (Visit(
                ip_address=request.META.get('REMOTE_ADDR',''),
                visitor_hash=visitor_hash,
                uri=uri,
                object_app=app_label
            ), True)

        if len(visit):
            visit[0].last_visit = Visit.objects.filter(visitor_hash=visitor_hash).aggregate(last_visit=models.Max('last_visit'))['last_visit']
            if not is_ignored(request, visit[0]):
                visit[0].last_visit = now()
                visit[0].visits += 1
                visit[0].save()

    def add_object_visit(self, request, obj):
        visitor_hash = gen_hash(request, obj._meta.app_label, obj.__class__.__name__, obj.id)
        if settings.VISITS_OBJECTS_AS_COUNTERS:
            visit = self.get_or_create(
                visitor_hash=visitor_hash,
                object_app=obj._meta.app_label,
                object_model=obj.__class__.__name__,
                object_id=obj.id,
                ip_address=request.META.get('REMOTE_ADDR', '')
            )
        else:
            visit = (Visit(
                visitor_hash=visitor_hash,
                object_app=obj._meta.app_label,
                object_model=obj.__class__.__name__,
                object_id=obj.id,
                ip_address=request.META.get('REMOTE_ADDR', '')
            ), True)

        if len(visit):
            visit[0].last_visit = Visit.objects.filter(visitor_hash=visitor_hash).aggregate(last_visit=models.Max('last_visit'))['last_visit']
            if not is_ignored(request, visit[0]):
                visit[0].last_visit = now()
                visit[0].visits += 1
                visit[0].save()

    def calculate(self, obj, uri, what=MAX):
        if obj:
            visits = self.filter(
                object_app=obj._meta.app_label,
                object_model=obj.__class__.__name__)
        elif uri:
            visits = self.filter(uri=uri).aggregate(Max("visits"))
        else:
            raise Exception("Parameters obj or uri must be specified")

        if what is MAX:
            return visits.aggregate(Max("visits"))
        elif what is MIN:
            return visits.aggregate(Min("visits"))
        elif what is AVG:
            return visits.aggregate(Avg("visits"))
        else:
            return None

class Visit(models.Model):
    visitor_hash = models.CharField(max_length=40, blank=True, null=True, db_index=True)
    uri = models.CharField(max_length=255, blank=True, null=True)
    ip_address = models.IPAddressField(blank=True, null=True, db_index=True)
    last_visit = models.DateTimeField(blank=True, null=True)
    visits = models.IntegerField(default=0)
    object_app = models.CharField(max_length=255)
    object_model = models.CharField(max_length=255)
    object_id = models.CharField(max_length=255)
    blocked = models.BooleanField(default=False)

    objects = VisitManager()

    def __unicode__(self):
        return self.visitor_hash

    class Meta:
        app_label = "visits"
        ordering = ('uri', 'object_model', 'object_id')
        verbose_name = _('visit')
        verbose_name_plural = _('visits')



from django.test import TestCase
from django.test.client import Client

class IsBotTest(TestCase):
    def setUp(self):
        self.client = Client()
    def test_i_am_a_bot(self):
        """
        Test by setting HTTP_USER_AGENT to Googlebot/2.1
        """
        user_agent = "Googlebot/2.1"
        response = self.client.get("/", **{ "HTTP_USER_AGENT": user_agent, "REMOTE_ADDR": "127.0.0.1" })
        self.assertTrue(response.context["request"].META.get("IS_BOT"))
    def test_i_am_a_normal_user(self):
        """
        Test by setting HTTP_USER_AGENT to Mozilla/5.0
        """
        user_agent = "Mozilla/5.0"
        response = self.client.get("/", **{ "HTTP_USER_AGENT": user_agent, "REMOTE_ADDR": "127.0.0.1" })
        self.assertFalse(response.context["request"].META.get("IS_BOT"))



# -*- coding: utf-8 -*-
from visits.utils import gen_hash

def request_meta(request):
    uri = request.META.get("PATH_INFO", None)
    ip = request.META.get('REMOTE_ADDR','')
    return {
        "visits_meta": {
            "request_path": uri,
            "visitor_hash": gen_hash(request, uri),
            "ip_address": ip,
        }
    }



# -*- coding: utf-8 -*-
from visits import settings
from visits.models import Visit
from visits.utils import is_ignored

class CounterMiddleware(object):
    """ Middleware for count uri visits. """

    def process_request(self, request):
        if settings.URI_WITH_GET_PARAMS:
            Visit.objects.add_uri_visit(request, request.get_full_path())
        else:
            Visit.objects.add_uri_visit(request, request.path_info)

class BotVisitorMiddleware(object):
    """ Middleware for count uri visits for bots. """

    def process_request(self, request):
        user_agent = request.META.get("HTTP_USER_AGENT", None)
        if user_agent in settings.BOTS_USER_AGENTS:
            request.META.setdefault("IS_BOT", True)
        else:
            request.META.setdefault("IS_BOT", False)



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Changing field 'Visit.uri'
        db.alter_column('visits_visit', 'uri', self.gf('django.db.models.fields.CharField')(max_length=255, null=True))


    def backwards(self, orm):
        
        # User chose to not deal with backwards NULL issues for 'Visit.uri'
        raise RuntimeError("Cannot reverse this migration. 'Visit.uri' and its values cannot be restored.")


    models = {
        'visits.visit': {
            'Meta': {'ordering': "('uri', 'object_model', 'object_id')", 'object_name': 'Visit'},
            'blocked': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'ip_address': ('django.db.models.fields.IPAddressField', [], {'db_index': 'True', 'max_length': '15', 'null': 'True', 'blank': 'True'}),
            'last_visit': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'object_app': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'object_id': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'object_model': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'uri': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'visitor_hash': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '40', 'null': 'True', 'blank': 'True'}),
            'visits': ('django.db.models.fields.IntegerField', [], {'default': '0'})
        }
    }

    complete_apps = ['visits']
